
object Test {

  def main (argv:Array[String]):Unit = run()

  def run ():Unit = {


    // Lists of Int

    val L1:List[Int] = List.cons(33,List.cons(66,List.cons(99,List.empty())))
    println("L1 = " + L1)
    println("Elements L1 = ")
    Stream.print[Int](L1)


    // Lists of String

    val L2:List[String] = 
      List.cons[String]("hello",
                        List.cons[String]("world",
                                          List.empty[String]))
    println("L2 = " + L2)
    println("Elements L2 = ")
    Stream.print[String](L2)


    // Note: we do not usually need to specify types when
    // calling parameterized methods -- the type checker
    // will often figure out the right type for us.
    //
    // So from now on, I will leave out those types when calling
    // parameterized methods, knowing that we can always toss them
    // in if the type checker gets confused
    //
    // However, I will still specify the type for declarations, 
    // since it documents what it is that we are expecting.


    // Trees of Int

    val T = BinTree

    val T1:BinTree[Int] = T.node(33,
				 T.node(66,
					T.node(99,T.empty(),T.empty()),
					T.empty()),
				 T.node(11,
					T.node(22,T.empty(),T.empty()),
					T.node(44,T.empty(),T.empty())))
    println("T1 = " + T1)
    println("Elements T1 = ")
    Stream.print(T1)


    // Lists of Int

    val L3:List[Int] = 
      List.cons(33,List.cons(66,List.cons(99,List.empty())))

    println("L3 = " + L3)
    println("Elements L3 = ")
    Stream.print(L3)
    println("Sum L3 = " + Stream.sumInt(L3))

    
    // Lists of PairInt

    val L4:List[PairInt] = 
      List.cons(PairInt.create(1,2),
		List.cons(PairInt.create(5,6),
			  List.cons(PairInt.create(8,9),
				    List.empty())))
    println("L4 = " + L4)
    println("Elements L4 = ")
    Stream.print(L4)
    println("Sum L4 = " + Stream.sumPairInt(L4))


    // Lists of AInt

    def i(n:Int):AInt = AInt.create(n)

    val L5:List[AInt] = 
      List.cons(i(33),List.cons(i(66),List.cons(i(99),List.empty())))

    println("L5 = " + L5)
    println("Elements L5 = ")
    Stream.print(L5)
    println("Sum L5 = " + Stream.sum(i(0),L5))


    // Trees of PairAInts

    def p (i:Int,j:Int):PairAInt = PairAInt.create(AInt.create(i),
						   AInt.create(j))
    val T2:BinTree[PairAInt] =
      T.node(p(1,2),
	     T.node(p(30,40),T.empty(),T.empty()),
	     T.node(p(500,600),
		    T.node(p(7000,8000),
			   T.empty(), T.empty()),
		    T.empty()))

    println("T2 = " + T2)
    println("Elements T2 = ")
    Stream.print(T2)
    println("Sum T2 = " + Stream.sum(p(0,0),T2))

    // Same thing, but with APair[AInt,AInt] instead of PairAInt
    // basically using a reusable form of pairs

    val p0:APair[AInt,AInt] = APair.create(i(0),i(0))
    val p1:APair[AInt,AInt] = APair.create(i(1),i(2))
    val p2:APair[AInt,AInt] = APair.create(i(30),i(40))
    val p3:APair[AInt,AInt] = APair.create(i(500),i(600))
    val p4:APair[AInt,AInt] = APair.create(i(7000),i(8000))
    
    val T3:BinTree[APair[AInt,AInt]] =
      T.node(p1,
	     T.node(p2,T.empty(),T.empty()),
	     T.node(p3,
		    T.node(p4,
			   T.empty(), T.empty()),
		    T.empty()))
    println("T3 = " + T3)
    println("Elements T3 = ")
    Stream.print(T3)
    println("Sum T3 = " + Stream.sum(p0,T3))


    // working with more interesting pairs

    val pp0:APair[AInt,APair[AInt,AInt]] = APair.create(i(0),p0)
    val pp1:APair[AInt,APair[AInt,AInt]] = APair.create(i(1),p1)
    val pp2:APair[AInt,APair[AInt,AInt]] = APair.create(i(2),p2)
    val pp3:APair[AInt,APair[AInt,AInt]] = APair.create(i(3),p3)
    val pp4:APair[AInt,APair[AInt,AInt]] = APair.create(i(4),p4)
    
    
    val T4:BinTree[APair[AInt,APair[AInt,AInt]]] =
      T.node(pp1,
	     T.node(pp2,T.empty(),T.empty()),
	     T.node(pp3,
		    T.node(pp4,
			   T.empty(), T.empty()),
		    T.empty()))
    println("T4 = " + T4)
    println("Elements T4 = ")
    Stream.print(T4)
    println("Sum T4 = " + Stream.sum(pp0,T4))

  }
}
